@@ -8,12 +8,16 @@ from ..utils.collections import dotdict
 
 
 class Model:
-    """The Model superclass runs simulations and manages inputs and outputs of all models."""
+    """The Model base class runs models, manages their outputs, parameters and more.
+    This class should serve as the base class for all implemented models.
+    """
 
     def __init__(self, integration, params):
         if hasattr(self, "name"):
             if self.name is not None:
                 assert isinstance(self.name, str), f"Model name is not a string."
+        else:
+            self.name = "Noname"
 
         assert integration is not None, "Model integration function not given."
         self.integration = integration
@@ -33,6 +37,10 @@ class Model:
 
         assert isinstance(self.default_output, str), "`default_output` must be a string."
 
+        # if no output_vars is set, it will be replaced by state_vars
+        if not hasattr(self, "output_vars"):
+            self.output_vars = self.state_vars
+
         # create output and state dictionary
         self.outputs = dotdict({})
         self.state = dotdict({})
@@ -76,7 +84,7 @@ class Model:
                         if not bold_input.shape[1] % self.boldModel.samplingRate_NDt == 0:
                             append = False
                             logging.warn(
-                                f"Output size {bold_input.shape[1]} is not a multiple of BOLD sample length { self.boldModel.samplingRate_NDt}, will not append data."
+                                f"Output size {bold_input.shape[1]} is not a multiple of BOLD sampling length { self.boldModel.samplingRate_NDt}, will not append data."
                             )
                         logging.debug(f"Simulating BOLD: boldModel.run(append={append})")
 
@@ -98,12 +106,57 @@ class Model:
         else:
             logging.warn("BOLD model not initialized, not simulating BOLD. Use `run(bold=True)`")
 
-    def checkChunkwise(self):
-        """Checks if the model fulfills requirements for chunkwise simulation. Throws errors if not."""
+    def checkChunkwise(self, chunksize):
+        """Checks if the model fulfills requirements for chunkwise simulation.
+        Checks whether the sampling rate for outputs fits to chunksize and duration.
+        Throws errors if not."""
         assert self.state_vars is not None, "State variable names not given."
         assert self.init_vars is not None, "Initial value variable names not given."
         assert len(self.state_vars) == len(self.init_vars), "State variables are not same length as initial values."
 
+        # throw a warning if the user is nasty
+        if int(self.params["duration"] / self.params["dt"]) % chunksize != 0:
+            logging.warning(
+                f"It is strongly advised to use a `chunksize` ({chunksize}) that is a divisor of `duration / dt` ({int(self.params['duration']/self.params['dt'])})."
+            )
+
+        # if `sampling_dt` is set, do some checks
+        if self.params.get("sampling_dt") is not None:
+            # sample_dt checks are required after setting chunksize
+            assert (
+                chunksize * self.params["dt"] >= self.params["sampling_dt"]
+            ), "`chunksize * dt` must be >= `sampling_dt`"
+
+            # ugly floating point modulo hack: instead of float1%float2==0, we do
+            # (float1/float2)%1==0
+            assert ((chunksize * self.params["dt"]) / self.params["sampling_dt"]) % 1 == 0, (
+                f"Chunksize {chunksize * self.params['dt']} must be divisible by sampling dt "
+                f"{self.params['sampling_dt']}"
+            )
+            assert (
+                (self.params["duration"] % (chunksize * self.params["dt"])) / self.params["sampling_dt"]
+            ) % 1 == 0, (
+                f"Last chunk of size {self.params['duration'] % (chunksize * self.params['dt'])} must be divisible by sampling dt "
+                f"{self.params['sampling_dt']}"
+            )
+
+    def setSamplingDt(self):
+        """Checks if sampling_dt is set correctly and sets self.`sample_every`
+        1) Check if sampling_dt is multiple of dt
+        2) Check if semplind_dt is greater than duration
+        """
+
+        if self.params.get("sampling_dt") is None:
+            self.sample_every = 1
+        elif self.params.get("sampling_dt") > 0:
+            assert self.params["sampling_dt"] >= self.params["dt"], "`sampling_dt` needs to be >= `dt`"
+            assert (
+                self.params["duration"] >= self.params["sampling_dt"]
+            ), "`sampling_dt` needs to be lower than `duration`"
+            self.sample_every = int(self.params["sampling_dt"] / self.params["dt"])
+        else:
+            raise ValueError(f"Can't handle `sampling_dt`={self.params.get('sampling_dt')}")
+
     def initializeRun(self, initializeBold=False):
         """Initialization before each run.
 
@@ -116,10 +169,12 @@ class Model:
         # length of the initial condition
         self.startindt = self.maxDelay + 1
 
+        # check dt / sampling_dt
+        self.setSamplingDt()
+
         # force bold if params['bold'] == True
-        if "bold" in self.params:
-            if self.params["bold"]:
-                initializeBold = True
+        if self.params.get("bold"):
+            initializeBold = True
         # set up the bold model, if it didn't happen yet
         if initializeBold and not self.boldInitialized:
             self.initializeBold()
@@ -134,7 +189,9 @@ class Model:
         append_outputs=None,
         continue_run=False,
     ):
-        """Main interfacing function to run a model.
+        """
+        Main interfacing function to run a model.
+
         The model can be run in three different ways:
         1) `model.run()` starts a new run.
         2) `model.run(chunkwise=True)` runs the simulation in chunks of length `chunksize`.
@@ -142,8 +199,6 @@ class Model:
 
         :param inputs: list of inputs to the model, must have the same order as model.input_vars. Note: no sanity check is performed for performance reasons. Take care of the inputs yourself.
         :type inputs: list[np.ndarray|]
-        :param continue_run: continue a simulation by using the initial values from a previous simulation
-        :type continue_run: bool
         :param chunkwise: simulate model chunkwise or in one single run, defaults to False
         :type chunkwise: bool, optional
         :param chunksize: size of the chunk to simulate in dt, defaults to 2s
@@ -152,6 +207,8 @@ class Model:
         :type bold: bool, optional
         :param append: append the chunkwise outputs to the outputs attribute, defaults to False, defaults to False
         :type append: bool, optional
+        :param continue_run: continue a simulation by using the initial values from a previous simulation
+        :type continue_run: bool
         """
         # TODO: legacy argument support
         if append_outputs is not None:
@@ -163,6 +220,9 @@ class Model:
 
         self.initializeRun(initializeBold=bold)
 
+        # enable chunkwise if chunksize is set
+        chunkwise = chunkwise if chunksize is None else True
+
         if chunkwise is False:
             self.integrate(append_outputs=append, simulate_bold=bold)
             if continue_run:
@@ -171,8 +231,10 @@ class Model:
         else:
             if chunksize is None:
                 chunksize = int(2000 / self.params["dt"])
+
             # check if model is safe for chunkwise integration
-            self.checkChunkwise()
+            # and whether sampling_dt is compatible with duration and chunksize
+            self.checkChunkwise(chunksize)
             if bold and not self.boldInitialized:
                 logging.warn(f"{self.name}: BOLD model not initialized, not simulating BOLD. Use `run(bold=True)`")
                 bold = False
@@ -206,9 +268,8 @@ class Model:
         self.storeOutputsAndStates(t, variables, append=append_outputs)
 
         # force bold if params['bold'] == True
-        if "bold" in self.params:
-            if self.params["bold"]:
-                simulate_bold = True
+        if self.params.get("bold"):
+            simulate_bold = True
 
         # bold simulation after integration
         if simulate_bold and self.boldInitialized:
@@ -250,7 +311,8 @@ class Model:
         self.state = dotdict({})
         self.outputs = dotdict({})
         # reinitialize bold model
-        self.initializeBold()
+        if self.params.get("bold"):
+            self.initializeBold()
 
     def storeOutputsAndStates(self, t, variables, append=False):
         """Takes the simulated variables of the integration and stores it to the appropriate model output and state object.
@@ -346,18 +408,24 @@ class Model:
         :return: maxmimum delay of the model in units of dt
         :rtype: int
         """
-        dt = self.params["dt"]
-        Dmat = self.params["lengthMat"]
+        dt = self.params.get("dt")
+        Dmat = self.params.get("lengthMat")
 
-        if "signalV" in self.params:
-            signalV = self.params["signalV"]
+        if Dmat is not None:
+            # divide Dmat by signalV
+            signalV = self.params.get("signalV") or 0
             if signalV > 0:
                 Dmat = Dmat / signalV
             else:
+                # if signalV is 0, eliminate delays
                 Dmat = Dmat * 0.0
 
-        Dmat_ndt = np.around(Dmat / dt)  # delay matrix in multiples of dt
-        max_global_delay = int(np.amax(Dmat_ndt))
+        # only if Dmat and dt exist, a global max delay can be computed
+        if Dmat is not None and dt is not None:
+            Dmat_ndt = np.around(Dmat / dt)  # delay matrix in multiples of dt
+            max_global_delay = int(np.amax(Dmat_ndt))
+        else:
+            max_global_delay = 0
         return max_global_delay
 
     def setStateVariables(self, name, data):
@@ -405,6 +473,14 @@ class Model:
             else:
                 raise ValueError(f"Don't know how to truncate data of shape {data.shape}.")
 
+        # subsample to sampling dt
+        if data.ndim == 1:
+            data = data[:: self.sample_every]
+        elif data.ndim == 2:
+            data = data[:, :: self.sample_every]
+        else:
+            raise ValueError(f"Don't know how to subsample data of shape {data.shape}.")
+
         # if the output is a single name (not dot.separated)
         if "." not in name:
             # append data
@@ -461,7 +537,7 @@ class Model:
         return lastOutput
 
     def __getitem__(self, key):
-        """Index outputs with a dictionary-like key"""
+        """Index outputs with a dictionary-like key, e.g., `model['rates_exc']`."""
         return self.getOutput(key)
 
     def getOutputs(self, group=""):
@@ -499,7 +575,9 @@ class Model:
 
     @property
     def output(self):
-        """Returns value of default output."""
+        """Returns value of default output as defined by `self.default_output`.
+        Note that all outputs are saved in the attribute `self.outputs`.
+        """
         assert self.default_output is not None, "Default output has not been set yet. Use `setDefaultOutput()`."
         return self.getOutput(self.default_output)
 
