@@ -2,10 +2,6 @@
 # names for excitatory and inhibitory masses
 EXC = "EXC"
 INH = "INH"
-# names for three levels of hierarchy
-NETWORK_NAME_STR = "net"
-NODE_NAME_STR = "node"
-MASS_NAME_STR = "mass"
 # names for matrices
 NODE_CONNECTIVITY = "local_connectivity"
 NODE_DELAYS = "local_delays"
