@@ -1,4 +1,5 @@
 import logging
+from itertools import chain, islice
 
 import numpy as np
 import symengine as se
@@ -6,16 +7,7 @@ from jitcdde import t as time_vector
 from jitcdde import y as state_vector
 
 from .backend import BackendIntegrator
-from .constants import (
-    EXC,
-    MASS_NAME_STR,
-    NETWORK_CONNECTIVITY,
-    NETWORK_DELAYS,
-    NETWORK_NAME_STR,
-    NODE_CONNECTIVITY,
-    NODE_DELAYS,
-    NODE_NAME_STR,
-)
+from .constants import EXC, NETWORK_CONNECTIVITY, NETWORK_DELAYS, NODE_CONNECTIVITY, NODE_DELAYS
 from .neural_mass import NeuralMass
 
 
@@ -41,6 +33,9 @@ class Node(BackendIntegrator):
     # default output of the node - all nodes need to have this variable defined
     default_output = None
 
+    # typical output variables of the model - these will be available in model instance
+    output_vars = []
+
     def __init__(self, neural_masses):
         """
         :param neural_masses: list of neural masses in this node
@@ -52,12 +47,15 @@ class Node(BackendIntegrator):
         # for all masses
         self.num_state_variables = sum([mass.num_state_variables for mass in self])
         self.num_noise_variables = sum([mass.num_noise_variables for mass in self])
+        assert len(self.noise_input) == self.num_noise_variables
         self.idx_state_var = None
         self.initialised = False
         assert self.default_output in self.state_variable_names[0]
+        assert all(var in self.state_variable_names[0] for var in self.output_vars)
         # mass types needs to be unique for all masses!
         mass_types = [mass.mass_type for mass in self]
         assert len(set(mass_types)) == len(mass_types), f"Mass types needs to be different: {mass_types}"
+        self._initial_state = None
 
     def __str__(self):
         """
@@ -68,6 +66,9 @@ class Node(BackendIntegrator):
             f": {', '.join([mass.name for mass in self])}"
         )
 
+    def __repr__(self):
+        return self.__str__()
+
     def describe(self):
         """
         Return description dict.
@@ -107,6 +108,19 @@ class Node(BackendIntegrator):
     def max_delay(self):
         return 0.0
 
+    @property
+    def noise_input(self):
+        return sum([mass.noise_input for mass in self], [])
+
+    @noise_input.setter
+    def noise_input(self, new_noise):
+        assert len(new_noise) == self.num_noise_variables
+        masses_noise_length = [mass.num_noise_variables for mass in self.masses]
+        new_noise = iter(new_noise)
+        for noise_chunk, mass in zip([list(islice(new_noise, 0, i)) for i in masses_noise_length], self.masses):
+            assert len(noise_chunk) == mass.num_noise_variables
+            mass.noise_input = noise_chunk
+
     def get_nested_params(self):
         """
         Return nested dictionary with parameters from all masses within this
@@ -116,10 +130,10 @@ class Node(BackendIntegrator):
         :rtype: dict
         """
         assert self.initialised
-        node_key = f"{NODE_NAME_STR}_{self.index}"
+        node_key = f"{self.label}_{self.index}"
         nested_dict = {node_key: {}}
         for mass in self:
-            mass_key = f"{MASS_NAME_STR}_{mass.index}"
+            mass_key = f"{mass.label}_{mass.index}"
             nested_dict[node_key][mass_key] = mass.params
         return nested_dict
 
@@ -138,8 +152,17 @@ class Node(BackendIntegrator):
         for mass in self:
             mass.init_mass(**kwargs)
         assert all(mass.initialised for mass in self)
+        self._initial_state = np.array(sum([mass.initial_state for mass in self], []))
         self.initialised = True
 
+    def _sanitize_update_params(self, params_dict):
+        """
+        If dictionary with parameters for update have one title level, trim this.
+        """
+        if len(params_dict) == 1 and self.label in next(iter(params_dict)):
+            params_dict = next(iter(params_dict.values()))
+        return params_dict
+
     def update_params(self, params_dict):
         """
         Update parameters of the node, i.e. recursively update all parameters of masses within this node.
@@ -148,8 +171,10 @@ class Node(BackendIntegrator):
             `get_nested_params`, i.e. nested dict
         :type params_dict: dict
         """
+        mass_labels = [mass.label for mass in self.masses]
+        params_dict = self._sanitize_update_params(params_dict)
         for mass_key, mass_params in params_dict.items():
-            if MASS_NAME_STR in mass_key:
+            if any(mass_label in mass_key for mass_label in mass_labels):
                 mass_index = self._get_index(mass_key)
                 assert mass_index == self.masses[mass_index].index
                 self.masses[mass_index].update_params(mass_params)
@@ -197,7 +222,20 @@ class Node(BackendIntegrator):
         """
         Return initial state of this node, i.e. sum of initial states of all masses.
         """
-        return np.array(sum([mass.initial_state for mass in self], [],))
+        return self._initial_state
+
+    @initial_state.setter
+    def initial_state(self, initial_state):
+        """
+        Manually set initial state - be sure what you are doing!
+
+        :param initial_state: vector representing the initial state, if 2D pass as nodes x time
+        :type initial_state: np.ndarray
+        """
+        assert isinstance(initial_state, np.ndarray)
+        assert initial_state.ndim in [1, 2]
+        assert initial_state.shape[0] == self.num_state_variables
+        self._initial_state = initial_state
 
     def all_couplings(self, mass_indices=None):
         """
@@ -270,7 +308,10 @@ class SingleCouplingExcitatoryInhibitoryNode(Node):
     default_network_coupling = {"network_exc_exc": 0.0}
 
     def __init__(
-        self, neural_masses, local_connectivity, local_delays=None,
+        self,
+        neural_masses,
+        local_connectivity,
+        local_delays=None,
     ):
         """
         :param neural_masses: list of neural masses in this node
@@ -326,7 +367,7 @@ class SingleCouplingExcitatoryInhibitoryNode(Node):
         :rtype: dict
         """
         nested_params = super().get_nested_params()
-        node_key = f"{NODE_NAME_STR}_{self.index}"
+        node_key = f"{self.label}_{self.index}"
         nested_params[node_key][NODE_CONNECTIVITY] = self.connectivity
         nested_params[node_key][NODE_DELAYS] = self.delays
 
@@ -360,6 +401,7 @@ class SingleCouplingExcitatoryInhibitoryNode(Node):
         Update params - also update local connectivity and local delays,
         then pass to base class.
         """
+        params_dict = self._sanitize_update_params(params_dict)
         local_connectivity = params_dict.pop(NODE_CONNECTIVITY, None)
         local_delays = params_dict.pop(NODE_DELAYS, None)
         if local_connectivity is not None and isinstance(local_connectivity, np.ndarray):
@@ -416,6 +458,9 @@ class Network(BackendIntegrator):
     # default output of the network - e.g. BOLD is computed from this
     default_output = None
 
+    # typical output variables of the model - these will be available in model instance
+    output_vars = []
+
     def __init__(self, nodes, connectivity_matrix, delay_matrix=None):
         """
         :param nodes: list of nodes in this network
@@ -433,12 +478,14 @@ class Network(BackendIntegrator):
         self.nodes = nodes
         self.num_state_variables = sum([node.num_state_variables for node in self])
         self.num_noise_variables = sum([node.num_noise_variables for node in self])
+        assert len(self.noise_input) == self.num_noise_variables
         assert connectivity_matrix.shape[0] == self.num_nodes
         if delay_matrix is None:
             delay_matrix = np.zeros_like(connectivity_matrix)
         assert connectivity_matrix.shape == delay_matrix.shape
         self.connectivity = connectivity_matrix
         self.delays = delay_matrix
+        self._initial_state = None
         self.initialised = False
 
         if self.default_output is None:
@@ -446,6 +493,7 @@ class Network(BackendIntegrator):
             assert len(default_output) == 1
             self.default_output = next(iter(default_output))
 
+        assert all(var in chain.from_iterable(self.state_variable_names) for var in self.output_vars)
         assert all(self.default_output in node_state_vars for node_state_vars in self.state_variable_names)
 
         self.init_network()
@@ -456,6 +504,9 @@ class Network(BackendIntegrator):
         """
         return f"Brain network {self.name} with {self.num_nodes} nodes"
 
+    def __repr__(self):
+        return self.__str__()
+
     def describe(self):
         """
         Return description dict.
@@ -501,7 +552,33 @@ class Network(BackendIntegrator):
         """
         Return initial state for whole network.
         """
-        return np.concatenate([node.initial_state for node in self], axis=0)
+        return self._initial_state
+
+    @initial_state.setter
+    def initial_state(self, initial_state):
+        """
+        Manually set initial state - be sure what you are doing!
+
+        :param initial_state: vector representing the initial state, if 2D pass as nodes x time
+        :type initial_state: np.ndarray
+        """
+        assert isinstance(initial_state, np.ndarray)
+        assert initial_state.ndim in [1, 2]
+        assert initial_state.shape[0] == self.num_state_variables
+        self._initial_state = initial_state
+
+    @property
+    def noise_input(self):
+        return sum([node.noise_input for node in self], [])
+
+    @noise_input.setter
+    def noise_input(self, new_noise):
+        assert len(new_noise) == self.num_noise_variables
+        nodes_noise_length = [node.num_noise_variables for node in self.nodes]
+        new_noise = iter(new_noise)
+        for noise_chunk, node in zip([list(islice(new_noise, 0, i)) for i in nodes_noise_length], self.nodes):
+            assert len(noise_chunk) == node.num_noise_variables
+            node.noise_input = noise_chunk
 
     @staticmethod
     def _strip_index(symbol_name):
@@ -546,9 +623,9 @@ class Network(BackendIntegrator):
         :rtype: dict
         """
         assert self.initialised
-        nested_dict = {NETWORK_NAME_STR: {}}
+        nested_dict = {self.label: {}}
         for node in self:
-            nested_dict[NETWORK_NAME_STR].update(node.get_nested_params())
+            nested_dict[self.label].update(node.get_nested_params())
         nested_dict[NETWORK_CONNECTIVITY] = self.connectivity
         nested_dict[NETWORK_DELAYS] = self.delays
         return nested_dict
@@ -568,6 +645,7 @@ class Network(BackendIntegrator):
         for node_idx, node in enumerate(self.nodes):
             node.init_node(start_idx_for_noise=node_idx * node.num_noise_variables)
         assert all(node.initialised for node in self)
+        self._initial_state = np.concatenate([node.initial_state for node in self], axis=0)
         self.initialised = True
 
     def update_params(self, params_dict):
@@ -578,8 +656,11 @@ class Network(BackendIntegrator):
         :param params_dict: new parameters for the network
         :type params_dict: dict
         """
+        node_labels = [node.label for node in self.nodes]
+        if len(params_dict) == 1 and self.label in next(iter(params_dict)):
+            params_dict = next(iter(params_dict.values()))
         for node_key, node_params in params_dict.items():
-            if NODE_NAME_STR in node_key:
+            if any(node_label in node_key for node_label in node_labels):
                 node_index = int(node_key.split("_")[-1])
                 assert node_index == self.nodes[node_index].index
                 self.nodes[node_index].update_params(node_params)
@@ -644,9 +725,14 @@ class Network(BackendIntegrator):
             # iterate over indices as `to`, hence rows
             for to_node in range(self.num_nodes):
                 # input at `to` x `from`
-                inputs[to_node, from_node] = state_vector(
-                    var_idx + node_var_idx, time=time_vector - self.delays[to_node, from_node],
-                )
+                # if None - no input
+                if node_var_idx is None:
+                    inputs[to_node, from_node] = 0.0
+                else:
+                    inputs[to_node, from_node] = state_vector(
+                        var_idx + node_var_idx,
+                        time=time_vector - self.delays[to_node, from_node],
+                    )
             var_idx += node.num_state_variables
 
         return inputs
@@ -660,7 +746,7 @@ class Network(BackendIntegrator):
         """
         return [(self.sync_symbols[f"{symbol}_{node_idx}"], 0.0) for node_idx in range(self.num_nodes)]
 
-    def _diffusive_coupling(self, within_node_idx, symbol, connectivity_multiplier=1.0):
+    def _diffusive_coupling(self, within_node_idx, symbol, connectivity=None):
         """
         Perform diffusive coupling on the network with given symbol, i.e.
             network_inp = SUM_idx(Cmat[to, idx] * (X[idx](t - Dmat) - X[to](t)))
@@ -670,16 +756,17 @@ class Network(BackendIntegrator):
         :type within_node_idx: list[int]|int
         :param symbol: which symbol to fill with the data
         :type symbol: str
-        :param connectivity_multiplier: multiplier for connectivity, either
-            scalar, or array broadcastable to connectivity
-        :type connectivity_multiplier: float|np.ndarray
+        :param connectivity: connectivity matrix - if None will use the network
+            connectivity from init
+        :type connectivity: np.ndarray
         """
         assert symbol in self.sync_variables
         if isinstance(within_node_idx, int):
             within_node_idx = [within_node_idx] * self.num_nodes
         assert self.num_nodes == len(within_node_idx)
         inputs = self._construct_input_matrix(within_node_idx)
-        connectivity = self.connectivity * connectivity_multiplier
+        connectivity = self.connectivity if connectivity is None else connectivity
+        assert connectivity.shape == (self.num_nodes, self.num_nodes)
         var_idx = 0
         helpers = []
         for node_idx, node_var_idx in zip(range(self.num_nodes), within_node_idx):
@@ -698,7 +785,7 @@ class Network(BackendIntegrator):
             var_idx += self.nodes[node_idx].num_state_variables
         return helpers
 
-    def _additive_coupling(self, within_node_idx, symbol, connectivity_multiplier=1.0):
+    def _additive_coupling(self, within_node_idx, symbol, connectivity=None):
         """
         Perform additive coupling on the network within given symbol, i.e.
             network_inp = SUM_idx(Cmat[to, idx] * X[idx](t - Dmat))
@@ -708,12 +795,14 @@ class Network(BackendIntegrator):
         :type within_node_idx: list[int]|int
         :param symbol: which symbol to fill with the data
         :type symbol: str
-        :param connectivity_multiplier: multiplier for connectivity, either
-            scalar, or array broadcastable to connectivity
-        :type connectivity_multiplier: float|np.ndarray
+        :param connectivity: connectivity matrix - if None will use the network
+            connectivity from init
+        :type connectivity: np.ndarray
         """
         assert symbol in self.sync_variables
-        connectivity = self.connectivity * connectivity_multiplier * self._construct_input_matrix(within_node_idx)
+        connectivity = self.connectivity if connectivity is None else connectivity
+        assert connectivity.shape == (self.num_nodes, self.num_nodes)
+        connectivity = connectivity * self._construct_input_matrix(within_node_idx)
         return [
             (
                 self.sync_symbols[f"{symbol}_{node_idx}"],
@@ -734,9 +823,15 @@ class Network(BackendIntegrator):
         """
         assert coupling_variable in self.coupling_symbols
         if coupling_type == "additive":
-            return self._additive_coupling(self.coupling_symbols[coupling_variable], f"network_{coupling_variable}",)
+            return self._additive_coupling(
+                self.coupling_symbols[coupling_variable],
+                f"network_{coupling_variable}",
+            )
         elif coupling_type == "diffusive":
-            return self._diffusive_coupling(self.coupling_symbols[coupling_variable], f"network_{coupling_variable}",)
+            return self._diffusive_coupling(
+                self.coupling_symbols[coupling_variable],
+                f"network_{coupling_variable}",
+            )
         elif coupling_type == "none":
             return self._no_coupling(f"network_{coupling_variable}")
         else:
