@@ -1,7 +1,11 @@
+from copy import deepcopy
+
 import numpy as np
 import symengine as se
 from jitcdde import y as state_vector
 
+from .....utils.stimulus import ModelInput
+
 
 class NeuralMass:
     """
@@ -38,6 +42,9 @@ class NeuralMass:
     # number of noise variables / inputs
     num_noise_variables = 0
 
+    # inputs to the mass - usually noise
+    _noise_input = []
+
     # names for the state variables to link them with results
     state_variable_names = []
 
@@ -78,7 +85,7 @@ class NeuralMass:
         :type seed: int|None
         """
         assert isinstance(params, dict)
-        self.params = params
+        self.params = deepcopy(params)
         self.seed = seed
         # used in determining portion of the full system's state vector
         self.idx_state_var = None
@@ -101,6 +108,9 @@ class NeuralMass:
             f"state variables: {', '.join(self.state_variable_names)}"
         )
 
+    def __repr__(self):
+        return self.__str__()
+
     def describe(self):
         """
         Return description dict.
@@ -137,6 +147,17 @@ class NeuralMass:
         )
         assert all(callable(callback[1]) for callback in callback_list)
 
+    @property
+    def noise_input(self):
+        return self._noise_input
+
+    @noise_input.setter
+    def noise_input(self, new_noise):
+        assert len(new_noise) == self.num_noise_variables
+        self._noise_input = new_noise
+        # update noise parameters
+        self._get_params_from_noise()
+
     def init_mass(self, start_idx_for_noise=None):
         """
         Initialise neural mass. Usually just initialise the state vector,
@@ -148,8 +169,15 @@ class NeuralMass:
             start_idx_for_noise = self.index
         if self.noise_input_idx is None:
             self.noise_input_idx = [start_idx_for_noise + i for i in range(self.num_noise_variables)]
+        assert len(self.noise_input) == self.num_noise_variables
+        assert all(isinstance(noise_process, ModelInput) for noise_process in self.noise_input), self.noise_input
+        self._get_params_from_noise()
         self.initialised = True
 
+    def _get_params_from_noise(self):
+        for i, noise_process in enumerate(self.noise_input):
+            self.params[f"noise_{i}"] = noise_process.get_params()
+
     def update_params(self, params_dict):
         """
         Update parameters of the mass.
@@ -158,9 +186,13 @@ class NeuralMass:
         :type params_dict: dict
         """
         assert isinstance(params_dict, dict)
-        self.params.update(params_dict)
+        self.params.update({k: v for k, v in params_dict.items() if "noise" not in k})
         # validate again
         self._validate_params()
+        for i, noise_process in enumerate(self.noise_input):
+            noise_process.update_params(params_dict.get(f"noise_{i}", {}))
+        # update noise params
+        self._get_params_from_noise()
 
     def _callbacks(self):
         """
@@ -188,7 +220,13 @@ class NeuralMass:
         Unwrap state vector into individual variables. Uses global
         `state_vector` from `jitc*de`.
         """
-        return [state_vector(i) for i in range(self.idx_state_var, self.idx_state_var + self.num_state_variables,)]
+        return [
+            state_vector(i)
+            for i in range(
+                self.idx_state_var,
+                self.idx_state_var + self.num_state_variables,
+            )
+        ]
 
     def _derivatives(self, coupling_variables=None):
         """
