@@ -1,6 +1,7 @@
 import numpy as np
 from jitcdde import input as system_input
 
+from ....utils.stimulus import OrnsteinUhlenbeckProcess
 from ..builder.base.network import Network, Node
 from ..builder.base.neural_mass import NeuralMass
 
@@ -39,6 +40,10 @@ class FitzHughNagumoMass(NeuralMass):
         "y_ext",
     ]
     required_couplings = ["network_x", "network_y"]
+    _noise_input = [
+        OrnsteinUhlenbeckProcess(mu=0.0, sigma=0.0, tau=5.0),
+        OrnsteinUhlenbeckProcess(mu=0.0, sigma=0.0, tau=5.0),
+    ]
 
     def __init__(self, params=None, seed=None):
         super().__init__(params=params or FHN_DEFAULT_PARAMS, seed=seed)
@@ -84,6 +89,7 @@ class FitzHughNagumoNode(Node):
 
     default_network_coupling = {"network_x": 0.0, "network_y": 0.0}
     default_output = "x"
+    output_vars = ["x", "y"]
 
     def __init__(self, params=None, seed=None):
         """
@@ -111,9 +117,14 @@ class FitzHughNagumoNetwork(Network):
     sync_variables = ["network_x", "network_y"]
     # define default coupling in FitzHugh-Nagumo network
     default_coupling = {"network_x": "diffusive", "network_y": "none"}
+    output_vars = ["x", "y"]
 
     def __init__(
-        self, connectivity_matrix, delay_matrix, mass_params=None, seed=None,
+        self,
+        connectivity_matrix,
+        delay_matrix,
+        mass_params=None,
+        seed=None,
     ):
         """
         :param connectivity_matrix: connectivity matrix for between nodes
@@ -142,7 +153,9 @@ class FitzHughNagumoNetwork(Network):
             nodes.append(node)
 
         super().__init__(
-            nodes=nodes, connectivity_matrix=connectivity_matrix, delay_matrix=delay_matrix,
+            nodes=nodes,
+            connectivity_matrix=connectivity_matrix,
+            delay_matrix=delay_matrix,
         )
         # get all coupling variables
         all_couplings = [mass.coupling_variables for node in self.nodes for mass in node.masses]
