@@ -1,29 +1,25 @@
 import datetime
-import os
 import logging
 import multiprocessing
-import sys
+import os
 
 import deap
-from deap import base
-from deap import creator
-from deap import tools
-
 import numpy as np
-import pypet as pp
 import pandas as pd
+import pypet as pp
+from deap import base, creator, tools
 
 from ...utils import paths as paths
 from ...utils import pypetUtils as pu
 from ...utils.parameterSpace import ParameterSpace
-
-from . import evolutionaryUtils as eu
+from ...utils.collections import unwrap_star_dotdict, BACKWARD_REPLACE
 from . import deapUtils as du
+from . import evolutionaryUtils as eu
 
 
 class Evolution:
-    """Evolutionary parameter optimization. This class helps you to optimize any function or model using an evlutionary algorithm. 
-    It uses the package `deap` and supports its builtin mating and selection functions as well as custom ones. 
+    """Evolutionary parameter optimization. This class helps you to optimize any function or model using an evlutionary algorithm.
+    It uses the package `deap` and supports its builtin mating and selection functions as well as custom ones.
     """
 
     def __init__(
@@ -90,7 +86,7 @@ class Evolution:
         :param PARENT_SELECT_P: Parent selection operator keyword arguments (for the default operator selRank, this defaults to `s` = 1.5 in Eiben&Smith p.81)
         :type PARENT_SELECT_P: dict, optional
 
-        :param individualGenerator: Function to generate initial individuals, defaults to du.randomParametersAdaptive     
+        :param individualGenerator: Function to generate initial individuals, defaults to du.randomParametersAdaptive
         """
 
         if weightList is None:
@@ -210,7 +206,7 @@ class Evolution:
         self.MUTATE_P = self.MUTATE_P if hasattr(self, "MUTATE_P") else {}
         self.SELECT_P = self.SELECT_P if hasattr(self, "SELECT_P") else {}
 
-        self.initDEAP(
+        self._initDEAP(
             self.toolbox,
             self.env,
             self.paramInterval,
@@ -224,8 +220,12 @@ class Evolution:
         )
 
         # set up pypet trajectory
-        self.initPypetTrajectory(
-            self.traj, self.paramInterval, self.POP_SIZE, self.NGEN, self.model,
+        self._initPypetTrajectory(
+            self.traj,
+            self.paramInterval,
+            self.POP_SIZE,
+            self.NGEN,
+            self.model,
         )
 
         # population history: dict of all valid individuals per generation
@@ -238,7 +238,7 @@ class Evolution:
     def run(self, verbose=False):
         """Run the evolution or continue previous evolution. If evolution was not initialized first
         using `runInitial()`, this will be done.
-        
+
         :param verbose: Print and plot state of evolution during run, defaults to False
         :type verbose: bool, optional
         """
@@ -251,7 +251,7 @@ class Evolution:
 
     def getIndividualFromTraj(self, traj):
         """Get individual from pypet trajectory
-        
+
         :param traj: Pypet trajectory
         :type traj: `pypet.trajectory.Trajectory`
         :return: Individual (`DEAP` type)
@@ -273,14 +273,18 @@ class Evolution:
         :params traj: Pypet trajectory with individual (traj.individual) or directly a deap.Individual
 
         :returns model: Model with the parameters of this individual.
-        
+
         :param traj: Pypet trajectory with individual (traj.individual) or directly a deap.Individual
         :type traj: `pypet.trajectory.Trajectory`
         :return: Model with the parameters of this individual.
         :rtype: `neurolib.models.model.Model`
         """
         model = self.model
-        model.params.update(self.individualToDict(self.getIndividualFromTraj(traj)))
+        # resolve star notation - MultiModel
+        individual_params = self.individualToDict(self.getIndividualFromTraj(traj))
+        if self.parameterSpace.star:
+            individual_params = unwrap_star_dotdict(individual_params, self.model, replaced_dict=BACKWARD_REPLACE)
+        model.params.update(individual_params)
         return model
 
     def getIndividualFromHistory(self, id):
@@ -300,7 +304,7 @@ class Evolution:
 
     def individualToDict(self, individual):
         """Convert an individual to a parameter dictionary.
-        
+
         :param individual: Individual (`DEAP` type)
         :type individual: `deap.creator.Individual`
         :return: Parameter dictionary of this individual
@@ -308,9 +312,9 @@ class Evolution:
         """
         return self.ParametersInterval(*(individual[: len(self.paramInterval)]))._asdict().copy()
 
-    def initPypetTrajectory(self, traj, paramInterval, POP_SIZE, NGEN, model):
+    def _initPypetTrajectory(self, traj, paramInterval, POP_SIZE, NGEN, model):
         """Initializes pypet trajectory and store all simulation parameters for later analysis.
-        
+
         :param traj: Pypet trajectory (must be already initialized!)
         :type traj: `pypet.trajectory.Trajectory`
         :param paramInterval: Parameter space, from ParameterSpace class
@@ -351,10 +355,12 @@ class Evolution:
         traj.f_add_parameter("id", 0, comment="Index of individual")
         traj.f_add_parameter("ind_len", 20, comment="Length of individual")
         traj.f_add_derived_parameter(
-            "individual", [0 for x in range(traj.ind_len)], "An indivudal of the population",
+            "individual",
+            [0 for x in range(traj.ind_len)],
+            "An indivudal of the population",
         )
 
-    def initDEAP(
+    def _initDEAP(
         self,
         toolbox,
         pypetEnvironment,
@@ -368,7 +374,7 @@ class Evolution:
         individualGenerator,
     ):
         """Initializes DEAP and registers all methods to the deap.toolbox
-        
+
         :param toolbox: Deap toolbox
         :type toolbox: deap.base.Toolbox
         :param pypetEnvironment: Pypet environment (must be initialized first!)
@@ -393,7 +399,10 @@ class Evolution:
         # need to create a lambda funciton because du.generateRandomParams wants an argument but
         # toolbox.register cannot pass an argument to it.
         toolbox.register(
-            "individual", deap.tools.initIterate, deap.creator.Individual, lambda: individualGenerator(paramInterval),
+            "individual",
+            deap.tools.initIterate,
+            deap.creator.Individual,
+            lambda: individualGenerator(paramInterval),
         )
         logging.info(f"Evolution: Individual generation: {individualGenerator}")
 
@@ -416,7 +425,7 @@ class Evolution:
         toolbox.register("select", selectionOperator)
         logging.info(f"Evolution: Selection operator: {selectionOperator}")
 
-    def evalPopulationUsingPypet(self, traj, toolbox, pop, gIdx):
+    def _evalPopulationUsingPypet(self, traj, toolbox, pop, gIdx):
         """Evaluate the fitness of the popoulation of the current generation using pypet
         :param traj: Pypet trajectory
         :type traj: `pypet.trajectory.Trajectory`
@@ -510,7 +519,7 @@ class Evolution:
         pop = pop or self.pop
         return [p for p in pop if np.isnan(p.fitness.values).any() or np.isinf(p.fitness.values).any()]
 
-    def tagPopulation(self, pop):
+    def _tagPopulation(self, pop):
         """Take a fresh population and add id's and attributes such as parameters that we can use later
 
         :param pop: Fresh population
@@ -544,10 +553,10 @@ class Evolution:
         ### Evaluate the initial population
         logging.info("Evaluating initial population of size %i ..." % len(self.pop))
         self.gIdx = 0  # set generation index
-        self.pop = self.tagPopulation(self.pop)
+        self.pop = self._tagPopulation(self.pop)
 
         # evaluate
-        self.pop = self.evalPopulationUsingPypet(self.traj, self.toolbox, self.pop, self.gIdx)
+        self.pop = self._evalPopulationUsingPypet(self.traj, self.toolbox, self.pop, self.gIdx)
 
         if self.verbose:
             eu.printParamDist(self.pop, self.paramInterval, self.gIdx)
@@ -566,8 +575,7 @@ class Evolution:
         self._t_end_initial_population = datetime.datetime.now()
 
     def runEvolution(self):
-        """Run the evolutionary optimization process for `NGEN` generations.
-        """
+        """Run the evolutionary optimization process for `NGEN` generations."""
         # Start evolution
         logging.info("Start of evolution")
         self._t_start_evolution = datetime.datetime.now()
@@ -579,12 +587,15 @@ class Evolution:
 
             logging.info("Replacing {} invalid individuals.".format(len(invalidpop)))
             newpop = self.toolbox.population(n=len(invalidpop))
-            newpop = self.tagPopulation(newpop)
+            newpop = self._tagPopulation(newpop)
 
             # ------- Create the next generation by crossover and mutation -------- #
             ### Select parents using rank selection and clone them ###
             offspring = list(
-                map(self.toolbox.clone, self.toolbox.selectParents(self.pop, self.POP_SIZE, **self.PARENT_SELECT_P),)
+                map(
+                    self.toolbox.clone,
+                    self.toolbox.selectParents(self.pop, self.POP_SIZE, **self.PARENT_SELECT_P),
+                )
             )
 
             ##### cross-over ####
@@ -599,19 +610,19 @@ class Evolution:
                 offspring[i].parentIds = offspring[i - 1].id, offspring[i].id
 
                 # delete id originally set from parents, needs to be deleted here!
-                # will be set later in tagPopulation()
+                # will be set later in _tagPopulation()
                 del offspring[i - 1].id, offspring[i].id
 
             ##### Mutation ####
             # Apply mutation
             du.mutateUntilValid(offspring, self.paramInterval, self.toolbox, MUTATE_P=self.MUTATE_P)
 
-            offspring = self.tagPopulation(offspring)
+            offspring = self._tagPopulation(offspring)
 
             # ------- Evaluate next generation -------- #
 
             self.pop = offspring + newpop
-            self.evalPopulationUsingPypet(self.traj, self.toolbox, offspring + newpop, self.gIdx)
+            self._evalPopulationUsingPypet(self.traj, self.toolbox, offspring + newpop, self.gIdx)
 
             # log individuals
             self.history[self.gIdx] = validpop + offspring + newpop  # self.getValidPopulation(self.pop)
@@ -648,9 +659,9 @@ class Evolution:
         self.traj.f_store()  # We switched off automatic storing, so we need to store manually
         self._t_end_evolution = datetime.datetime.now()
 
-        self.buildEvolutionTree()
+        self._buildEvolutionTree()
 
-    def buildEvolutionTree(self):
+    def _buildEvolutionTree(self):
         """Builds a genealogy tree that is networkx compatible.
 
         Plot the tree using:
@@ -678,7 +689,7 @@ class Evolution:
 
     def info(self, plot=True, bestN=5, info=True, reverse=False):
         """Print and plot information about the evolution and the current population
-        
+
         :param plot: plot a plot using `matplotlib`, defaults to True
         :type plot: bool, optional
         :param bestN: Print summary of `bestN` best individuals, defaults to 5
@@ -708,12 +719,14 @@ class Evolution:
             except:
                 logging.warning("Could not plot progress, is this a previously saved simulation?")
             eu.plotPopulation(
-                self, plotScattermatrix=True, save_plots=self.trajectoryName, color=self.plotColor,
+                self,
+                plotScattermatrix=True,
+                save_plots=self.trajectoryName,
+                color=self.plotColor,
             )
 
     def plotProgress(self, reverse=False):
-        """Plots progress of fitnesses of current evolution run
-        """
+        """Plots progress of fitnesses of current evolution run"""
         eu.plotProgress(self, reverse=reverse)
 
     def saveEvolution(self, fname=None):
@@ -729,9 +742,9 @@ class Evolution:
         logging.info(f"Saving evolution to {fname}")
 
     def loadEvolution(self, fname):
-        """Load evolution from previously saved simulatoins. 
-        
-        Example usage: 
+        """Load evolution from previously saved simulatoins.
+
+        Example usage:
         ```
         evaluateSimulation = lambda x: x # the funciton can be ommited, that's why we define a lambda here
         pars = ParameterSpace(['a', 'b'], # should be same as previously saved evolution
@@ -751,7 +764,10 @@ class Evolution:
         # parameter space is not saved correctly in dill, don't know why
         # that is why we recreate it using the values of
         # the parameter space in the dill
-        pars = ParameterSpace(evolution.parameterSpace.parameterNames, evolution.parameterSpace.parameterValues,)
+        pars = ParameterSpace(
+            evolution.parameterSpace.parameterNames,
+            evolution.parameterSpace.parameterValues,
+        )
 
         evolution.parameterSpace = pars
         evolution.paramInterval = evolution.parameterSpace.named_tuple
@@ -812,7 +828,7 @@ class Evolution:
         return df
 
     def dfPop(self, outputs=False):
-        """Returns a `pandas` DataFrame of the current generation's population parameters. 
+        """Returns a `pandas` DataFrame of the current generation's population parameters.
         This object can be further used to easily analyse the population.
         :return: Pandas DataFrame with all individuals and their parameters
         :rtype: `pandas.core.frame.DataFrame`
@@ -885,7 +901,7 @@ class Evolution:
     def loadResults(self, filename=None, trajectoryName=None):
         """Load results from a hdf file of a previous evolution and store the
         pypet trajectory in `self.traj`
-        
+
         :param filename: hdf filename of the previous run, defaults to None
         :type filename: str, optional
         :param trajectoryName: Name of the trajectory in the hdf file to load. If not given, the last one will be loaded, defaults to None
@@ -896,14 +912,13 @@ class Evolution:
         self.traj = pu.loadPypetTrajectory(filename, trajectoryName)
 
     def getScores(self):
-        """Returns the scores of the current valid population
-        """
+        """Returns the scores of the current valid population"""
         validPop = self.getValidPopulation(self.pop)
         return np.array([pop.fitness.score for pop in validPop])
 
     def getScoresDuringEvolution(self, traj=None, drop_first=True, reverse=False):
         """Get the scores of each generation's population.
-        
+
         :param traj: Pypet trajectory. If not given, the current trajectory is used, defaults to None
         :type traj: `pypet.trajectory.Trajectory`, optional
         :param drop_first: Drop the first (initial) generation. This can be usefull because it can have a different size (`POP_INIT_SIZE`) than the succeeding populations (`POP_SIZE`) which can make data handling tricky, defaults to True
