@@ -1,21 +1,20 @@
-import multiprocessing
-import datetime
-import os
-import logging
-import pathlib
 import copy
-import psutil
+import datetime
+import logging
+import multiprocessing
+import os
+import pathlib
 
 import numpy as np
 import pandas as pd
-import h5py
+import psutil
 import pypet
 import tqdm
+import xarray as xr
 
 from ...utils import paths
 from ...utils import pypetUtils as pu
-
-from ...utils.collections import dotdict
+from ...utils.collections import dotdict, flat_dict_to_nested, flatten_nested_dict, unwrap_star_dotdict
 
 
 class BoxSearch:
@@ -23,18 +22,17 @@ class BoxSearch:
     Paremter box search for a given model and a range of parameters.
     """
 
-    def __init__(
-        self, model=None, parameterSpace=None, evalFunction=None, filename=None, saveAllModelOutputs=False,
-    ):
+    def __init__(self, model=None, parameterSpace=None, evalFunction=None, filename=None, saveAllModelOutputs=False):
         """Either a model has to be passed, or an evalFunction. If an evalFunction
-        is passed, then the evalFunction will be called and the model is accessible to the 
-        evalFunction via `self.getModelFromTraj(traj)`. The parameters of the current 
+        is passed, then the evalFunction will be called and the model is accessible to the
+        evalFunction via `self.getModelFromTraj(traj)`. The parameters of the current
         run are accible via `self.getParametersFromTraj(traj)`.
 
         If no evaluation function is passed, then the model is simulated using `Model.run()`
         for every parameter.
 
-        :param model: Model to run for each parameter (or model to pass to the evaluation funciton if an evaluation function is used), defaults to None
+        :param model: Model to run for each parameter (or model to pass to the evaluation funciton if an evaluation
+            function is used), defaults to None
         :type model: `neurolib.models.model.Model`, optional
         :param parameterSpace: Parameter space to explore, defaults to None
         :type parameterSpace: `neurolib.utils.parameterSpace.ParameterSpace`, optional
@@ -42,12 +40,14 @@ class BoxSearch:
         :type evalFunction: function, optional
         :param filename: HDF5 storage file name, if left empty, defaults to ``exploration.hdf``
         :type filename: str
-        :param saveAllModelOutputs: If True, save all outputs of model, else only default output of the model will be saved. Note: if saveAllModelOutputs==False and the model's parameter model.params['bold']==True, then BOLD output will be saved as well, defaults to False
+        :param saveAllModelOutputs: If True, save all outputs of model, else only default output of the model will be
+            saved. Note: if saveAllModelOutputs==False and the model's parameter model.params['bold']==True, then BOLD
+            output will be saved as well, defaults to False
         :type saveAllModelOutputs: bool
         """
         self.model = model
         if evalFunction is None and model is not None:
-            self.evalFunction = self.runModel
+            self.evalFunction = self._runModel
         elif evalFunction is not None:
             self.evalFunction = evalFunction
 
@@ -60,7 +60,8 @@ class BoxSearch:
         self.parameterSpace = parameterSpace
         self.exploreParameters = parameterSpace.dict()
 
-        # todo: use random ICs for every explored point or rather reuse the ones that are generated at model initialization
+        # TODO: use random ICs for every explored point or rather reuse the ones that are generated at model
+        # initialization
         self.useRandomICs = False
 
         filename = filename or "exploration.hdf"
@@ -70,11 +71,13 @@ class BoxSearch:
 
         # bool to check whether pypet was initialized properly
         self.initialized = False
-        self.initializeExploration(self.filename)
+        self._initializeExploration(self.filename)
 
-    def initializeExploration(self, filename="exploration.hdf"):
+        self.results = None
+
+    def _initializeExploration(self, filename="exploration.hdf"):
         """Initialize the pypet environment
-        
+
         :param filename: hdf filename to store the results in , defaults to "exploration.hdf"
         :type filename: str, optional
         """
@@ -109,13 +112,17 @@ class BoxSearch:
         if self.model is not None:
             # if a model is specified, use the default parameter of the
             # model to initialize pypet
-            self.addParametersToPypet(self.traj, self.model.params)
+            self._addParametersToPypet(self.traj, self.model.params)
         else:
             # else, use a random parameter of the parameter space
-            self.addParametersToPypet(self.traj, self.parameterSpace.getRandom(safe=True))
+            self._addParametersToPypet(self.traj, self.parameterSpace.getRandom(safe=True))
 
         # Tell pypet which parameters to explore
         self.pypetParametrization = pypet.cartesian_product(self.exploreParameters)
+        # explicitely add all parameters within star notation, hence unwrap star notation into actual params names
+        if self.parameterSpace.star:
+            assert self.model is not None, "With star notation, model cannot be None"
+            self.pypetParametrization = unwrap_star_dotdict(self.pypetParametrization, self.model)
         self.nRuns = len(self.pypetParametrization[list(self.pypetParametrization.keys())[0]])
         logging.info(f"Number of parameter configurations: {self.nRuns}")
 
@@ -125,10 +132,10 @@ class BoxSearch:
         logging.info("BoxSearch: Environment initialized.")
         self.initialized = True
 
-    def addParametersToPypet(self, traj, params):
+    def _addParametersToPypet(self, traj, params):
         """This function registers the parameters of the model to Pypet.
         Parameters can be nested dictionaries. They are unpacked and stored recursively.
-        
+
         :param traj: Pypet trajectory to store the parameters in
         :type traj: `pypet.trajectory.Trajectory`
         :param params: Parameter dictionary
@@ -151,14 +158,10 @@ class BoxSearch:
 
         addParametersRecursively(traj, params, [])
 
-    # TODO: remove legacy
-    def saveOutputsToPypet(self, outputs, traj):
-        return self.saveToPypet(outputs, traj)
-
     def saveToPypet(self, outputs, traj):
         """This function takes simulation results in the form of a nested dictionary
         and stores all data into the pypet hdf file.
-        
+
         :param outputs: Simulation outputs as a dictionary.
         :type outputs: dict
         :param traj: Pypet trajectory
@@ -182,10 +185,10 @@ class BoxSearch:
         savestr = "results.$."
         makeSaveStringForPypet(value, savestr)
 
-    def runModel(self, traj):
+    def _runModel(self, traj):
         """If not evaluation function is given, we assume that a model will be simulated.
         This function will be called by pypet directly and therefore wants a pypet trajectory as an argument
-        
+
         :param traj: Pypet trajectory
         :type traj: `pypet.trajectory.Trajectory`
         """
@@ -193,6 +196,8 @@ class BoxSearch:
             logging.warn("Random initial conditions not implemented yet")
         # get parameters of this run from pypet trajectory
         runParams = self.getParametersFromTraj(traj)
+        if self.parameterSpace.star:
+            runParams = flatten_nested_dict(flat_dict_to_nested(runParams)["parameters"])
 
         # set the parameters for the model
         self.model.params.update(runParams)
@@ -204,9 +209,9 @@ class BoxSearch:
         # run it
         self.model.run(**runKwargs)
         # save outputs
-        self.saveModelOutputsToPypet(traj)
+        self._saveModelOutputsToPypet(traj)
 
-    def saveModelOutputsToPypet(self, traj):
+    def _saveModelOutputsToPypet(self, traj):
         # save all data to the pypet trajectory
         if self.saveAllModelOutputs:
             # save all results from exploration
@@ -214,7 +219,11 @@ class BoxSearch:
         else:
             # save only the default output
             self.saveToPypet(
-                {self.model.default_output: self.model.output, "t": self.model.outputs["t"],}, traj,
+                {
+                    self.model.default_output: self.model.output,
+                    "t": self.model.outputs["t"],
+                },
+                traj,
             )
             # save BOLD output
             # if "bold" in self.model.params:
@@ -223,8 +232,8 @@ class BoxSearch:
             if "BOLD" in self.model.outputs:
                 self.saveToPypet(self.model.outputs["BOLD"], traj)
 
-    def validatePypetParameters(self, runParams):
-        """Helper to handle None's in pypet parameters 
+    def _validatePypetParameters(self, runParams):
+        """Helper to handle None's in pypet parameters
         (used for random number generator seed)
 
         :param runParams: parameters as returned by traj.parameters.f_to_dict()
@@ -239,14 +248,15 @@ class BoxSearch:
 
     def getParametersFromTraj(self, traj):
         """Returns the parameters of the current run as a (dot.able) dictionary
-        
+
         :param traj: Pypet trajectory
         :type traj: `pypet.trajectory.Trajectory`
         :return: Parameter set of the current run
         :rtype: dict
         """
-        runParams = self.traj.parameters.f_to_dict(short_names=True, fast_access=True)
-        runParams = self.validatePypetParameters(runParams)
+        # DO NOT use short names for star notation dicts
+        runParams = self.traj.parameters.f_to_dict(short_names=not self.parameterSpace.star, fast_access=True)
+        runParams = self._validatePypetParameters(runParams)
         return dotdict(runParams)
 
     def getModelFromTraj(self, traj):
@@ -273,16 +283,21 @@ class BoxSearch:
 
     def loadResults(self, all=True, filename=None, trajectoryName=None, pypetShortNames=True, memory_cap=95.0):
         """Load results from a hdf file of a previous simulation.
-        
-        :param all: Load all simulated results into memory, which will be available as the `.results` attribute. Can use a lot of RAM if your simulation is large, please use this with caution. , defaults to True
+
+        :param all: Load all simulated results into memory, which will be available as the `.results` attribute. Can
+            use a lot of RAM if your simulation is large, please use this with caution. , defaults to True
         :type all: bool, optional
         :param filename: hdf file name in which results are stored, defaults to None
         :type filename: str, optional
-        :param trajectoryName: Name of the trajectory inside the hdf file, newest will be used if left empty, defaults to None
+        :param trajectoryName: Name of the trajectory inside the hdf file, newest will be used if left empty, defaults
+            to None
         :type trajectoryName: str, optional
-        :param pypetShortNames: Use pypet short names as keys for the results dictionary. Use if you are experiencing errors due to natural naming collisions.
+        :param pypetShortNames: Use pypet short names as keys for the results dictionary. Use if you are experiencing
+            errors due to natural naming collisions.
         :type pypetShortNames: bool
-        :param memory_cap: Percentage memory cap between 0 and 100. If `all=True` is used, a memory cap can be set to avoid filling up the available RAM. Example: use `memory_cap = 95` to avoid loading more data if memory is at 95% use, defaults to 95
+        :param memory_cap: Percentage memory cap between 0 and 100. If `all=True` is used, a memory cap can be set to
+            avoid filling up the available RAM. Example: use `memory_cap = 95` to avoid loading more data if memory is
+            at 95% use, defaults to 95
         :type memory_cap: float, int, optional
         """
 
@@ -329,9 +344,11 @@ class BoxSearch:
     def aggregateResultsToDfResults(self, arrays=True, fillna=False):
         """Aggregate all results in to dfResults dataframe.
 
-        :param arrays: Load array results (like timeseries) if True. If False, only load scalar results, defaults to True
+        :param arrays: Load array results (like timeseries) if True. If False, only load scalar results, defaults to
+            True
         :type arrays: bool, optional
-        :param fillna: Fill nan results (for example if they're not returned in a subset of runs) with zeros, default to False
+        :param fillna: Fill nan results (for example if they're not returned in a subset of runs) with zeros, default
+            to False
         :type fillna: bool, optional
         """
         nan_value = np.nan
@@ -355,7 +372,7 @@ class BoxSearch:
                 # only save floats, ints and arrays
                 if isinstance(value, (float, int, np.ndarray)):
                     # save 1-dim arrays
-                    if isinstance(value, np.ndarray) and arrays == True:
+                    if isinstance(value, np.ndarray) and arrays:
                         # to save a numpy array, convert column to object type
                         if key not in self.dfResults:
                             self.dfResults[key] = None
@@ -374,10 +391,11 @@ class BoxSearch:
 
     def loadDfResults(self, filename=None, trajectoryName=None):
         """Load results from a previous simulation.
-        
+
         :param filename: hdf file name in which results are stored, defaults to None
         :type filename: str, optional
-        :param trajectoryName: Name of the trajectory inside the hdf file, newest will be used if left empty, defaults to None
+        :param trajectoryName: Name of the trajectory inside the hdf file, newest will be used if left empty, defaults
+            to None
         :type trajectoryName: str, optional
         """
         # chose HDF file to load
@@ -389,11 +407,111 @@ class BoxSearch:
 
         # create pandas dataframe of all runs with parameters as keys
         logging.info("Creating `dfResults` dataframe ...")
-        niceParKeys = [p.split(".")[-1] for p in exploredParameters.keys()]
+        niceParKeys = [p[11:] for p in exploredParameters.keys()]
+        if not self.parameterSpace:
+            niceParKeys = [p.split(".")[-1] for p in niceParKeys]
         self.dfResults = pd.DataFrame(columns=niceParKeys, dtype=object)
         for nicep, p in zip(niceParKeys, exploredParameters.keys()):
             self.dfResults[nicep] = exploredParameters[p].f_get_range()
 
+    @staticmethod
+    def _filterDictionaryBold(filt_dict, bold):
+        """Filters result dictionary: either keeps ONLY BOLD results, or remove
+        BOLD results.
+
+        :param filt_dict: dictionary to filter for BOLD keys
+        :type filt_dict: dict
+        :param bold: whether to remove BOLD keys (bold=False) or keep only BOLD
+            keys (bold=True)
+        :return: filtered dict, without or only BOLD keys
+        :rtype: dict
+        """
+        filt_dict = copy.deepcopy(filt_dict)
+        if bold:
+            return {k: v for k, v in filt_dict.items() if "BOLD" in k}
+        else:
+            return {k: v for k, v in filt_dict.items() if "BOLD" not in k}
+
+    def _getCoordsFromRun(self, run_dict, bold=False):
+        """Find coordinates of a single run - time, output and space dimensions.
+
+        :param run_dict: dictionary with run results
+        :type run_dict: dict
+        :param bold: whether to do only BOLD or without BOLD results
+        :type bold: bool
+        :return: dictionary of coordinates for xarray
+        :rtype: dict
+        """
+        run_dict = copy.deepcopy(run_dict)
+        run_dict = self._filterDictionaryBold(run_dict, bold=bold)
+        timeDictKey = ""
+        if "t" in run_dict:
+            timeDictKey = "t"
+        else:
+            for k in run_dict:
+                if k.startswith("t"):
+                    timeDictKey = k
+                    logging.info(f"Assuming {k} to be the time axis.")
+                    break
+        assert len(timeDictKey) > 0, "No time array found (starting with t) in model output."
+        t = run_dict[timeDictKey].copy()
+        del run_dict[timeDictKey]
+        return timeDictKey, {
+            "output": list(run_dict.keys()),
+            "space": list(range(next(iter(run_dict.values())).shape[0])),
+            "time": t,
+        }
+
+    def xr(self, bold=False):
+        """
+        Return `xr.Dataset` from the exploration results.
+
+        :param bold: if True, will load and return only BOLD output
+        :type bold: bool
+        """
+        assert self.results is not None, "Run `loadResults()` first to populate the results"
+        assert len(self.results) == len(self.dfResults)
+        # create intrisinsic dims for one run
+        timeDictKey, run_coords = self._getCoordsFromRun(self.results[0], bold=bold)
+        dataarrays = []
+        orig_search_coords = pypet.cartesian_product(self.exploreParameters)
+        for runId, run_result in self.results.items():
+            # take exploration coordinates for this run
+            expl_coords = {k: v[runId] for k, v in orig_search_coords.items()}
+            outputs = []
+            run_result = self._filterDictionaryBold(run_result, bold=bold)
+            for key, value in run_result.items():
+                if key == timeDictKey:
+                    continue
+                outputs.append(value)
+            # create DataArray for run only - we need to add exploration coordinates
+            data_temp = xr.DataArray(
+                np.stack(outputs), dims=["output", "space", "time"], coords=run_coords, name="exploration"
+            )
+            expand_coords = {}
+            # iterate exploration coordinates
+            for k, v in expl_coords.items():
+                # if single values, just assing
+                if isinstance(v, (str, float, int)):
+                    expand_coords[k] = [v]
+                # if arrays, check whether they can be sqeezed into one value
+                elif isinstance(v, np.ndarray):
+                    if np.unique(v).size == 1:
+                        # if yes, just assing that one value
+                        expand_coords[k] = [float(np.unique(v))]
+                    else:
+                        # if no, sorry - coordinates cannot be array
+                        raise ValueError("Cannot squeeze coordinates")
+            # assing exploration coordinates to the DataArray
+            dataarrays.append(data_temp.expand_dims(expand_coords))
+
+        # finally, combine all arrays into one
+        combined = xr.combine_by_coords(dataarrays)["exploration"]
+        if self.parameterSpace.star:
+            combined.attrs = {k: list(self.model.params[k].keys()) for k in orig_search_coords.keys()}
+
+        return combined
+
     def getRun(self, runId, filename=None, trajectoryName=None, pypetShortNames=True):
         """Load the simulated data of a run and its parameters from a pypetTrajectory.
 
@@ -401,7 +519,7 @@ class BoxSearch:
         :type runId: int
 
         :return: Dictionary with simulated data and parameters of the run.
-        :type return: dict        
+        :type return: dict
         """
         # chose HDF file to load
         filename = self.HDF_FILE or filename
@@ -442,8 +560,7 @@ class BoxSearch:
         return self.results[runId] if hasattr(self, "results") else self.getRun(runId)
 
     def info(self):
-        """Print info about the current search.
-        """
+        """Print info about the current search."""
         now = datetime.datetime.now().strftime("%Y-%m-%d-%HH-%MM-%SS")
         print(f"Exploration info ({now})")
         print(f"HDF name: {self.HDF_FILE}")
