@@ -168,7 +168,9 @@ def plot_rates(model):
     plt.plot(model.t[model.t < plt_until], model.output[:, model.t < plt_until].T, lw=0.5)
 
 
-def plot_brain(model, ds, color=None, size=None, title=None, cbar=True, cmap="RdBu", clim=None):
+def plot_brain(
+    model, ds, color=None, size=None, title=None, cbar=True, cmap="RdBu", clim=None, cbarticks=None, cbarticklabels=None
+):
     """Dump and easy wrapper around the brain plotting function.
 
     :param color: colors of nodes, defaults to None
@@ -180,6 +182,9 @@ def plot_brain(model, ds, color=None, size=None, title=None, cbar=True, cmap="Rd
     s = Brainplot(ds.Cmat, model.output, fps=10, darkmode=False)
     s.cmap = plt.get_cmap(cmap)
 
+    if color is None:
+        color = np.ones(ds.Cmat.shape[0])
+
     dpi = 300
     fig = plt.figure(dpi=dpi)
     ax = plt.gca()
@@ -197,8 +202,9 @@ def plot_brain(model, ds, color=None, size=None, title=None, cbar=True, cmap="Rd
         # some weird scaling of the color to a size
         def norm(what):
             what = what.copy()
-            what -= np.min(what)
-            what /= np.max(what)
+            if np.min(what) < np.max(what):
+                what -= np.min(what)
+                what /= np.max(what)
             return what
 
         node_size = list(np.exp((norm(color) + 2) * 2))
@@ -212,10 +218,13 @@ def plot_brain(model, ds, color=None, size=None, title=None, cbar=True, cmap="Rd
         raise ValueError(f"node_size too big: {np.max(node_size)}")
     s.update(0, ax, node_color=color, node_size=node_size, clear=False)
     if cbar:
-        cbaxes = fig.add_axes([0.68, 0.1, 0.015, 0.7])
+        # cbaxes = fig.add_axes([0.68, 0.1, 0.015, 0.7])
+        cbaxes = fig.add_axes([0.75, 0.1, 0.015, 0.7])
         sm = plt.cm.ScalarMappable(cmap=s.cmap, norm=plt.Normalize(vmin=s.vmin, vmax=s.vmax))
-        cbar = plt.colorbar(sm, cbaxes)
+        cbar = plt.colorbar(sm, cbaxes, ticks=cbarticks)
         cbar.ax.tick_params(labelsize=16)
+        if cbarticklabels:
+            cbar.ax.set_yticklabels(cbarticklabels)
 
 
 # other plotting
@@ -377,11 +386,23 @@ def plot_degree_duration_scatterplot(model, states, ds, lingres=False, color_dow
     degrees = [(np.sum(ds.Cmat, axis=1))[i] / np.max(np.sum(ds.Cmat, axis=1)) for i in range(ds.Cmat.shape[0])]
 
     ax.scatter(
-        degrees, area_uptimes, s=14, c=color_up, edgecolor="black", linewidth=0.5, label="up-state",
+        degrees,
+        area_uptimes,
+        s=14,
+        c=color_up,
+        edgecolor="black",
+        linewidth=0.5,
+        label="up-state",
     )
 
     ax.scatter(
-        degrees, area_downtimes, s=14, c=color_down, edgecolor="black", linewidth=0.5, label="down-state",
+        degrees,
+        area_downtimes,
+        s=14,
+        c=color_down,
+        edgecolor="black",
+        linewidth=0.5,
+        label="down-state",
     )
     if lingres:
         plot_linregress(degrees, area_downtimes, kwargs={"c": color_down, "zorder": -2})
@@ -415,7 +436,7 @@ def plot_transition_phases(node_mean_phases_down, node_mean_phases_up, atlas):
     ipsilateral_mean_up_phase = [(i + k) / 2 for i, k in zip(node_mean_phases_up[0::2], node_mean_phases_up[1::2])]
     # ipsilateral_names = [i[:-2] for i, k in zip(atlas.names()[0::2], atlas.names()[1::2])]
     ipsilateral_names = [
-        f"{i[:-2]} ({nr*2+1}, {nr*2+2})" for nr, (i, k) in enumerate(zip(atlas.names()[0::2], atlas.names()[1::2]))
+        f"{i[:-2]} ({nr*2}, {nr*2+1})" for nr, (i, k) in enumerate(zip(atlas.names()[0::2], atlas.names()[1::2]))
     ]
     # clean up names
     for i in range(len(ipsilateral_names)):
@@ -452,9 +473,15 @@ def plot_transition_phases(node_mean_phases_down, node_mean_phases_up, atlas):
 
 def plot_transition_phases_against_eachother(node_mean_phases_down, node_mean_phases_up):
     figsize = (2, 2)
+    figsize = np.multiply(figsize, 0.75)
     plt.figure(figsize=figsize)
     plt.scatter(
-        node_mean_phases_down, node_mean_phases_up, s=8, edgecolor="black", linewidth=0.5, c="lightgray",
+        node_mean_phases_down,
+        node_mean_phases_up,
+        s=8,
+        edgecolor="black",
+        linewidth=0.5,
+        c="lightgray",
     )
     plot_linregress(node_mean_phases_down, node_mean_phases_up, kwargs={"c": "k", "alpha": 0.5, "zorder": -2})
     plt.xlabel("Mean down-phase")
@@ -474,7 +501,7 @@ def plot_state_brain_areas(model, states, atlas, ds, color_down="C0", color_up="
     ipsilateral_uptimes_diff = [(i - k) for i, k in zip(area_uptimes[0::2], area_uptimes[1::2])]
 
     ipsilateral_names = [
-        f"{i[:-2]} ({nr*2+1}, {nr*2+2})" for nr, (i, k) in enumerate(zip(atlas.names()[0::2], atlas.names()[1::2]))
+        f"{i[:-2]} ({nr*2}, {nr*2+1})" for nr, (i, k) in enumerate(zip(atlas.names()[0::2], atlas.names()[1::2]))
     ]
     # clean up names
     for i in range(len(ipsilateral_names)):
@@ -539,7 +566,7 @@ def plot_involvement_durations(model, states, involvement, nbins=6, legend=False
     figsize = np.multiply(figsize, 0.75)
     plt.figure(figsize=figsize)
     up_bin_means, bin_edges, _ = binned_statistic(invs[1], lens[1], bins=nbins, range=(0, 1))
-    plt.bar(bin_edges[:-1], up_bin_means, width=0.04, edgecolor="k", color="C0", label="up-state")
+    plt.bar(bin_edges[:-1], up_bin_means[::-1], width=0.04, edgecolor="k", color="C0", label="up-state")
 
     down_bin_means, bin_edges, _ = binned_statistic(invs[0], lens[0], bins=nbins, range=(0, 1))
     plt.bar(bin_edges[:-1] + 0.05, down_bin_means, width=0.04, edgecolor="k", color="C1", label="down-state")
@@ -547,6 +574,11 @@ def plot_involvement_durations(model, states, involvement, nbins=6, legend=False
         plt.legend(fontsize=8)
 
     plt.xticks([0, 0.5, 1], [0, 50, 100])
+    axs = plt.gca()
+    axs.spines["right"].set_visible(False)
+    axs.spines["top"].set_visible(False)
+    axs.spines["left"].set_visible(False)
+    plt.gca().tick_params(axis="both", direction="out", which="major", bottom=True, left=True)
     plt.xlabel("Involvement [%]")
     plt.ylabel("Duration [ms]")
     plt.tight_layout()
@@ -570,8 +602,9 @@ def plot_state_durations(model, states, legend=True, alpha=1.0):
     dt_to_s = model.params.dt / 1000
     flat_ups = [u * dt_to_s for up in ups for u in up]
     flat_downs = [d * dt_to_s for down in downs for d in down]
-    figsize = (3, 2.0)
-    figsize = np.multiply(figsize, 0.75)
+    # figsize = (3, 2.0)
+    figsize = (2, 1.7)
+    # figsize = np.multiply(figsize, 0.75)
     plt.figure(figsize=figsize)
     plt.hist(flat_ups, color="C0", label="up", edgecolor="k", linewidth=0.75)
     plt.hist(flat_downs, color="C1", label="down", edgecolor="k", linewidth=0.75, alpha=alpha)
@@ -579,40 +612,71 @@ def plot_state_durations(model, states, legend=True, alpha=1.0):
         plt.legend(fontsize=12, frameon=False)
     plt.gca().set_yscale("log")
     plt.gca().xaxis.set_major_locator(plt.MaxNLocator(4))
-    plt.ylabel("Count")
+    plt.ylabel("Log-probability")
+    plt.yticks([])
     plt.xlabel("Duration [s]")
+    plt.gca().tick_params(axis="x", direction="out", which="major", bottom=True, left=True)
+    ax = plt.gca()
+    ax.spines["right"].set_visible(False)
+    ax.spines["top"].set_visible(False)
+    ax.spines["left"].set_visible(False)
+    # ax.spines["bottom"].set_visible(False)
+
     plt.tight_layout()
     set_axis_size(*figsize)
 
 
-def plot_involvement_distribution(model, involvement, remove_xticks=True):
-    figsize = (3, 2)
-    figsize = np.multiply(figsize, 0.75)
+def plot_involvement_distribution(model, involvement, remove_xticks=True, color_localglobal=False):
+    # figsize = (3, 2)
+    figsize = (2, 1.7)
+    # figsize = np.multiply(figsize, 0.75)
     plt.figure(figsize=figsize)
-    plt.hist(involvement * 100, bins=10, density=True, rwidth=0.8, edgecolor="k", color="C1")
+    N, bins, patches = plt.hist(involvement * 100, bins=12, density=True, rwidth=0.8, edgecolor="k", color="C1")
+
+    if color_localglobal:
+        for i in range(0, len(patches) // 2):
+            patches[i].set_facecolor("C0")
+        for i in range(len(patches) // 2, len(patches)):
+            patches[i].set_facecolor("C1")
+
     plt.xticks([0, 50, 100])
     if remove_xticks:
         plt.yticks([])
+    else:
+        y_vals = plt.gca().get_yticks()
+        plt.gca().set_yticklabels(["{:3.0f}".format(x * 100) for x in y_vals])
     plt.ylabel("Probability")
     plt.xlabel("Involvement [%]")
+    plt.gca().tick_params(axis="x", direction="out", which="major", bottom=True, left=True)
     plt.xlim([0, 100])
+    ax = plt.gca()
+    ax.spines["right"].set_visible(False)
+    ax.spines["top"].set_visible(False)
+    ax.spines["left"].set_visible(False)
+    # ax.spines["bottom"].set_visible(False)
+    max_y = plt.gca().get_ylim()[1] * 1.1
+    plt.ylim(0, max_y * 1.1)
+    # plt.vlines([50], 0, max_y, linestyles="--", colors="#363638", alpha=0.9)
+    plt.text(25, max_y * 0.95, "local", fontsize=14, color="C0", ha="center")
+    plt.text(75, max_y * 0.95, "global", fontsize=14, color="C1", ha="center")
     plt.tight_layout()
     set_axis_size(*figsize)
 
 
-def plot_involvement_mean_amplitude(model, involvement, skip=300, lingress=True):
+def plot_involvement_mean_amplitude(model, involvement, skip=300, lingress=True, invert=False):
     figsize = (3, 2)
     figsize = np.multiply(figsize, 0.75)
     plt.figure(figsize=figsize)
     rates = np.mean(model.output, axis=0)[::skip]
+    plot_involvement = involvement[::skip] * 100 if not invert else ((1 - involvement[::skip]) * 100)
     plt.scatter(
-        involvement[::skip] * 100,
+        plot_involvement,
         rates,
         # scipy.stats.zscore(np.mean(model.output, axis=0)[::skip]),
-        s=4,
-        edgecolor="black",
+        s=10,
+        edgecolor="w",
         linewidth=0.2,
-        c="C1",
+        c="C1" if not invert else "C0",
         alpha=0.7,
     )
     # plt.gca().axhline(0, linestyle="--", lw=1, color="#ABB2BF", zorder=-1)
@@ -623,9 +687,55 @@ def plot_involvement_mean_amplitude(model, involvement, skip=300, lingress=True)
 
     if lingress:
         slope, intercept, r_value, p_value, std_err = plot_linregress(
-            involvement[::skip] * 100, rates, kwargs={"c": "k", "alpha": 0.75}
+            plot_involvement, rates, kwargs={"c": "k", "alpha": 0.75}
         )
-        plt.text(50, np.max(rates) * 0.75, f"$R^2={r_value**2:0.2f}$")
+        plt.text(50 if not invert else 25, np.max(rates) * 0.75, f"$R^2={r_value**2:0.2f}$")
+
+    axs = plt.gca()
+    axs.spines["right"].set_visible(False)
+    axs.spines["top"].set_visible(False)
+    axs.spines["left"].set_visible(False)
+    axs.spines["bottom"].set_visible(False)
+    plt.gca().tick_params(axis="both", direction="out", which="major", bottom=True, left=True)
+
+    plt.tight_layout()
+    set_axis_size(*figsize)
+
+
+def plot_involvement_nodewise_amplitude(model, involvement, skip=300, lingress=True, invert=False):
+    figsize = (3, 2)
+    figsize = np.multiply(figsize, 0.75)
+    plt.figure(figsize=figsize)
+    rates = np.mean(model.output, axis=0)[::skip]
+    plot_involvement = involvement[::skip] * 100 if not invert else ((1 - involvement[::skip]) * 100)
+
+    for r in model.output:
+        plt.scatter(
+            plot_involvement,
+            r[::skip],
+            s=1,
+            edgecolor="w",
+            linewidth=0.2,
+            c="C1" if not invert else "C0",
+            alpha=0.7,
+        )
+        if lingress:
+            slope, intercept, r_value, p_value, std_err = plot_linregress(
+                involvement * 100 if not invert else (1 - involvement) * 100,
+                r,
+                kwargs={"c": "k", "lw": 1, "alpha": 0.25},
+            )
+    plt.xlim(0, 100)
+    plt.xticks([0, 50, 100])
+    plt.xlabel("Involvement [%]")
+    plt.ylabel("Rate [Hz]")
+
+    axs = plt.gca()
+    axs.spines["right"].set_visible(False)
+    axs.spines["top"].set_visible(False)
+    axs.spines["left"].set_visible(False)
+    axs.spines["bottom"].set_visible(False)
+    plt.gca().tick_params(axis="both", direction="out", which="major", bottom=True, left=True)
 
     plt.tight_layout()
     set_axis_size(*figsize)
@@ -633,13 +743,13 @@ def plot_involvement_mean_amplitude(model, involvement, skip=300, lingress=True)
 
 def get_state_lengths(xs):
     """
-    Get's the length of successive elements in a list. 
+    Get's the length of successive elements in a list.
     Useful for computing inter-spike-intervals (ISI) or the length
     of states.
-    
+
     Example: get_state_lengths([0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 1, 0])
     Returns: [(0, 4), (1, 4), (0, 2), (1, 1), (0, 1)]
-    
+
     :param xs: Input list with successive states
     :type xs: list
     :return: List of (state, length) tuples
@@ -655,8 +765,7 @@ def get_state_lengths(xs):
 
 
 def get_updown_lengths(durations):
-    """Returns the length of all up- and down-states for each node
-    """
+    """Returns the length of all up- and down-states for each node"""
     ups = []
     downs = []
     for i, l in enumerate(durations):
@@ -814,9 +923,9 @@ def plot_phases_and_involvement(model, involvement, phases, states):
     for i, downs in enumerate(down_transitions):
         axs[0].vlines(np.where(downs)[0] * model.params.dt / 1000, -3, 0, lw=0.2, alpha=1, color="C1")
     # annotate vlines
-    axs[0].text(7.7, 7.0, "up-transitions", fontsize=8)
+    axs[0].text(7.7, 7.0, "down-to-up", fontsize=8)
     axs[0].vlines(7.6, 6.8, 9.0, lw=1, color="C0")
-    axs[0].text(7.7, 4.4, "down-transitions", fontsize=8)
+    axs[0].text(7.7, 4.4, "up-to-down", fontsize=8)
     axs[0].vlines(7.6, 4.0, 6.2, lw=1, color="C1")
 
     # axs[0].fill_between(model.t/1000, phases, 0, where=phases<0, zorder=-2, color='C0', alpha=0.8, label='$\phi < 0$')
@@ -827,7 +936,7 @@ def plot_phases_and_involvement(model, involvement, phases, states):
     axs[0].legend(fontsize=8, frameon=False, loc="upper center", bbox_to_anchor=(0.5, 1.1), ncol=1)
 
     axs[1].plot(model.t / 1000, np.mean(model.output, axis=0), c="k", alpha=1, lw=1, label="Mean rate")
-    axs[1].plot(model.t / 1000, involvement * 30, lw=1, c="C3", label="Involvement")
+    axs[1].plot(model.t / 1000, involvement * 30, lw=1, c="C1", label="Involvement")
 
     axs[1].set_xlim(0, 10)
     axs[1].set_yticks([])
@@ -851,6 +960,7 @@ def plot_ts(
     plot_nodes=[0],
     plot_alpha=1,
     plot_mean_background=False,
+    mean_alpha=0.3,
     shift_mean=False,
     adaptation=False,
     stimulus=False,
@@ -929,7 +1039,7 @@ def plot_ts(
             mean_signal /= np.max(mean_signal)
             mean_signal *= np.abs(np.max(plot_output) - np.min(plot_output)) / 4
             mean_signal = mean_signal + np.max(plot_output) * 1.1
-        axs.plot(plot_t, mean_signal, alpha=0.3, lw=lw, c="k", zorder=-10, label="Mean brain")
+        axs.plot(plot_t, mean_signal, alpha=mean_alpha, lw=lw, c="k", zorder=10, label="Mean brain")
 
     # if a stimulus was present
     if legend and stimulus and model.params["ext_exc_current"].ndim > 0:
@@ -1055,7 +1165,7 @@ def plot_matrix(mat, cbarlabel, cbar=True, ylabels=True, plotlog=False):
         # cbar = plt.colorbar(im, cbaxes)
         # cbar.set_ticks([0, 1])
         # cbar.ax.tick_params(width=0, labelsize=10)
-        cbar.set_label(label=cbarlabel, size=10, labelpad=-1)
+        cbar.set_label(label=cbarlabel, labelpad=5)
     plt.tight_layout()
     set_axis_size(*figsize)
 
@@ -1168,7 +1278,7 @@ def plot_fc_over_time(model, ds, bold_transient=0):
 def barplot_annotate_brackets(
     num1, num2, data, center, height, yerr=None, dh=0.05, barh=0.05, fs=None, maxasterix=None
 ):
-    """ 
+    """
     Annotate barplot with p-values.
 
     :param num1: number of left bar to put bracket over
@@ -1230,9 +1340,7 @@ def barplot_annotate_brackets(
 
 
 def plot_powerspectra(f, powerspecs, mean_power, savename=None, logx=False, logy=True, maxfr=None):
-    """Plottnig EEG power spectra (RESULT-data-eeg-powerspectrum.ipynb)
-
-    """
+    """Plottnig EEG power spectra (RESULT-data-eeg-powerspectrum.ipynb)"""
     plt.figure(figsize=(3, 3), dpi=300)
 
     # plt.title(f"Sleep stage: {stage}")
@@ -1273,4 +1381,3 @@ def plot_powerspectra(f, powerspecs, mean_power, savename=None, logx=False, logy
         plt.savefig(save_fname)
 
     # plt.show()
-
